'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.default = Auth;

var _events = require('events');

var _user = require('./user');

var _user2 = _interopRequireDefault(_user);

var _util = require('util');

var _https = require('https');

var _https2 = _interopRequireDefault(_https);

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var URL = 'https://auth.simperium.com/1';

function Auth(appId, appSecret) {
	this.appId = appId;
	this.appSecret = appSecret;
}

(0, _util.inherits)(Auth, _events.EventEmitter);

Auth.prototype.authorize = function (username, password) {
	var body = JSON.stringify({ username: username, password: password }),
	    promise = this.request('authorize/', body);

	return promise;
};

Auth.prototype.create = function (username, password) {
	var body = JSON.stringify({ username: username, password: password }),
	    promise = this.request('create/', body);

	return promise;
};

Auth.prototype.getUrlOptions = function (path) {
	var options = _url2.default.parse((0, _util.format)('%s/%s/%s', URL, this.appId, path));
	return Object.assign(options, { method: 'POST', headers: { 'X-Simperium-API-Key': this.appSecret } });
};

Auth.prototype.request = function (endpoint, body) {
	var _this = this;

	return new Promise(function (resolve, reject) {
		var req = _https2.default.request(_this.getUrlOptions(endpoint), function (res) {
			var responseData = '';

			res.on('data', function (data) {
				responseData += data.toString();
			});

			res.on('end', function () {
				var user;

				try {
					user = _user2.default.fromJSON(responseData);
				} catch (e) {
					return reject(new Error(responseData));
				}
				_this.emit('authorize', user);
				resolve(user);
			});
		});

		req.on('error', function (e) {
			reject(e);
		});

		req.end(body);
	});
};