'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.default = Bucket;

var _events = require('events');

var _util = require('util');

var _nodeUuid = require('node-uuid');

var _nodeUuid2 = _interopRequireDefault(_nodeUuid);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function Bucket(name, storeProvider) {
	_events.EventEmitter.call(this);
	this.name = name;
	this.store = storeProvider(this);
	this.isIndexing = false;
}

(0, _util.inherits)(Bucket, _events.EventEmitter);

Bucket.prototype.reload = function () {
	this.emit('reload');
};

Bucket.prototype.add = function (object, callback) {
	var id = _nodeUuid2.default.v4();
	return this.update(id, object, callback);
};

Bucket.prototype.get = function (id, callback) {
	return this.store.get(id, callback);
};

Bucket.prototype.update = function (id, data, options, callback) {
	if (typeof options === 'function') {
		callback = options;
	}
	return this.store.update(id, data, this.isIndexing, callback);
};

Bucket.prototype.touch = function (id, callback) {
	var _this = this;

	return this.store.get(id, function (e, object) {
		if (e) return callback(e);
		_this.update(object.id, object.data, callback);
	});
};

Bucket.prototype.remove = function (id, callback) {
	return this.store.remove(id, callback);
};

Bucket.prototype.find = function (query, callback) {
	return this.store.find(query, callback);
};

Bucket.prototype.getRevisions = function (id, callback) {
	// Overridden in Channel
	callback(new Error('Failed to fetch revisions for' + id));
};