'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.apply = exports.modify = exports.transform = exports.diff = exports.compressChanges = exports.buildChange = exports.type = undefined;

var _nodeUuid = require('node-uuid');

var _nodeUuid2 = _interopRequireDefault(_nodeUuid);

var _jsondiff2 = require('../jsondiff');

var _jsondiff3 = _interopRequireDefault(_jsondiff2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var changeTypes = {
	MODIFY: 'M',
	REMOVE: '-',
	ADD: '+'
};

var _jsondiff = (0, _jsondiff3.default)({ list_diff: false }),
    object_diff = _jsondiff.object_diff,
    transform_object_diff = _jsondiff.transform_object_diff,
    apply_object_diff = _jsondiff.apply_object_diff;

exports.type = changeTypes;
exports.buildChange = buildChange;
exports.compressChanges = compressChanges;
exports.diff = object_diff;
exports.transform = rebase;
exports.modify = modify;
exports.apply = apply_diff;


function modify(id, version, patch) {
	return { o: changeTypes.MODIFY, id: id, ccid: _nodeUuid2.default.v4(), v: patch };
}

function buildChange(type, id, object, ghost) {
	return buildChangeFromOrigin(type, id, ghost.version, object, ghost.data);
}

function buildChangeFromOrigin(type, id, version, target, origin) {
	var changeData = {
		o: type,
		id: id,
		ccid: _nodeUuid2.default.v4()
	};

	// Remove operations have no source version or diff
	if (type === changeTypes.REMOVE) return changeData;

	if (version > 0) changeData.sv = version;

	changeData.v = object_diff(origin, target);

	return changeData;
}

function compressChanges(changes, origin) {
	var modified;

	if (changes.length === 0) {
		return {};
	}

	if (changes.length === 1) {
		return changes[0].v;
	}

	modified = changes.reduce(function (from, change) {
		// deletes when, any changes after a delete are ignored
		if (from === null) return null;
		if (from.o === changeTypes.REMOVE) return null;
		return apply_object_diff(from, change.v);
	}, origin);

	if (modified === null) return null;

	return object_diff(origin, modified);
}

function rebase(local_diff, remote_diff, origin) {
	return transform_object_diff(local_diff, remote_diff, origin);
}

function apply_diff(patch, object) {
	return apply_object_diff(object, patch);
}